<?php


class Paypal
{
    protected $business;
    protected $cert_id;
    protected $button;
    protected $mode;
    protected $return;
    protected $cancel_return;
    protected $notify_url;
    protected $dirCertif;
    protected $nameBoutique;



    public function __construct()
    {
        $this->business = self::$config['business'];
        $this->cert_id = self::$config['cert_id'];
        $this->button = self::$config['button'];
        $this->mode = self::$config['mode'];
        $this->return = self::$config['return'];
        $this->cancel_return = self::$config['cancel_return'];
        $this->notify_url = self::$config['notify_url'];
        $this->dirCertif = self::$config['dirCertif'];
        $this->nameBoutique = self::$config['nameBoutique'];
    }

    static protected $config = null;

    /**
     * Init la config de paypal
     *
     * @param array $config :
     *  - business : email du compte
     *  - cert_id : certid du certificat SSL
     *  - button : url de l'image du boutton paypal
     *  - mode : sendox | production
     *  - return : url de retour en cas de succee
     *  - cancel_return : url de retour en cas d'annulation / echec
     *  - notify_url : url de notification
     *  - dirCertif chemin des certificat
     *  - nameBoutique : Nom de la boutique
     */
    static public function init($config = array()) {
        self::$config = $config;
    }

    /**
     * @param int $idReference
     * @param float $amount
     * @return mixed
     */
    public function request($idReference, $amount)
    {

        if(empty($_SESSION['commande']['livraison']['adresse_telephone'])) {
            $_SESSION['commande']['livraison']['adresse_telephone'] = '';
        }

        // encrypt button
        $buttonParams = array(
            "cmd"			=> "_xclick",
            "business" 		=> $this->business,
            "cert_id"		=> $this->cert_id,
            "charset"		=> "utf-8",
            "item_name"		=> $this->nameBoutique,
            "quantity"		=> 1,
            "amount"		=> $amount,
            "undefined_quantity" => 0,
            "no_note" => 1,
            "no_shipping" => 2,
            "cn" => 'Retour a la boutique',
            "cbt" => 'Retour a la boutique',
            "currency_code"	=> 'EUR',
            "return"		=> $this->return,
            "cancel_return"	=> $this->cancel_return,
            "notify_url"	=> $this->notify_url,
            "custom"		=> $idReference,

            // adresse de livraison
            'address1' =>  $_SESSION['commande']['livraison']['adresse_rue'],
            'address2' => $_SESSION['commande']['livraison']['adresse_rue2'],
            'city' => $_SESSION['commande']['livraison']['adresse_ville'],
            'country' =>  $_SESSION['commande']['livraison']['pays_code'],
            'first_name' => $_SESSION['commande']['livraison']['adresse_prenom'],
            'last_name' => $_SESSION['commande']['livraison']['adresse_nom'],
            'state' => '',
            'zip' => $_SESSION['commande']['livraison']['adresse_cp'],
            'night_phone_b' =>  $_SESSION['commande']['livraison']['adresse_telephone'],

            // adresse de facturation
            'billing_address1' =>  $_SESSION['commande']['facturation']['adresse_rue'],
            'billing_address2' =>  $_SESSION['commande']['facturation']['adresse_rue2'],
            'billing_city' =>  $_SESSION['commande']['facturation']['adresse_ville'],
            'billing_country' =>  $_SESSION['commande']['facturation']['pays_code_iso_2'],
            'billing_first_name' =>  $_SESSION['commande']['facturation']['adresse_prenom'],
            'billing_last_name' =>  $_SESSION['commande']['facturation']['adresse_nom'],
            'billing_state' =>  '',
            'billing_zip' =>  $_SESSION['commande']['facturation']['adresse_cp'],

            'address_override' => 1,
            'buyer_email' =>  $_SESSION['client_email'],
            'email' =>  $_SESSION['client_email'],
            'showShippingAddress' => 'false',
            'showBillingAddress' => 'false',
            'showBillingPhone' => 'false',
            'showBillingEmail' => 'false'
        );

        $envURL = ($this->mode == 'sandbox') ? "https://www.sandbox.paypal.com" : "https://www.paypal.com";

        $buttonReturn = \EWPServices::encryptButton(	$buttonParams,
            $this->dirCertif.'mon-certpub.pem',
            $this->dirCertif.'ma-clepriv.pem',
            '',
            $this->dirCertif.'paypal-cert.pem',
            $envURL,
            $this->button);

        if(!$buttonReturn["status"]) {
            exit;
        }

        return $buttonReturn["encryptedButton"];
    }

    /**
     * @param array $request
     * @return mixed
     */
    public function response($request)
    {
        if(empty($request) && !empty($_POST)) {
            $request = $_POST;
        }
        if(empty($request) && !empty($_GET)) {
            $request = $_GET;
        }
        if(empty($request)) {
            return false;
        }

        // lire la publication du systeme PayPal et ajouter 'cmd'
        $req = 'cmd=_notify-validate';
        foreach ($request as $key => $value) {
            $value = urlencode(stripslashes($value));
            $req .= "&$key=$value";
        }

        // renvoyer au systeme PayPal pour validation
        $header = "POST /cgi-bin/webscr HTTP/1.1\r\n";
        $header .= "Host: www.paypal.com\r\n";
        $header .= "Content-Type: application/x-www-form-urlencoded\r\n";
        $header .= "Content-Length: " . strlen($req) . "\r\n\r\n";
        $fp = fsockopen ('www.paypal.com', 80, $errno, $errstr, 30);
        // affecter les variables publiees aux variables locales
        $order_id = $request['custom'];

        // on verifie que l'on vient pas de notre compte
        if ($_POST['business'] != $this->business) {
            if ($request['business'] != $this->business) {
                $tpe = new Tpe;
                $tpe->paiementKO($order_id, $request);
                return true;
            }
        }

        if (!$fp) {
            return false;
        } else {
            fputs ($fp, $header . $req);
            while (!feof($fp)) {

                $res = fgets ($fp, 1024);
                $res = trim($res);

                if (strcmp ($res, "VERIFIED") == 0 && $request['payment_status']=="Completed") {
                    // VALID
                    $tpe = new Tpe;
                    $tpe->paiementOK($order_id, $request);
                    return true;
                }
                elseif (strcmp ($res, "INVALID") == 0) {
                    // INVALID
                    $tpe = new Tpe;
                    $tpe->paiementKO($order_id, $request);
                    return true;
                }
            }
            fclose($fp);
        }
    }
}